/****************************************************************************
; *
; *     File        : d3dprj.h
; *
; *     Description : Direct3D 8 Wrapper
; *
; *     Copyright  Realtech VR 2001
; *
; *     Permission to use, copy, modify, distribute and sell this software
; *     and its documentation for any purpose is hereby granted without fee, 
; *     provided that the above copyright notice appear in all copies and
; *     that both that copyright notice and this permission notice appear
; *     in supporting documentation.  Realtech makes no representations
; *     about the suitability of this software for any purpose.
; *     It is provided "as is" without express or implied warranty.
; *
; ***************************************************************************/
#ifndef _D3DPRJ_HH
#define _D3DPRJ_HH
#include <assert.h>
#include <string.h>

#ifdef _WIN32
#include <stdlib.h>
#include <float.h>
#include <limits.h>
#include <math.h>

#undef _WIN32
#define __WIN32__
#endif // This hack will disable _WIN32 compilation mode

#include "d3d8.h"


#if defined __WIN32__
#ifdef _USE_MCD
#include "win32/mcd.h"	// MCD Wrapper	
#else
#include <GL/gl.h>		// Defined for Win32 applications	
#endif
#elif defined __BEOS__
#include <GL/gl.h>		// Defined for BeOS applications
#elif defined __MACOS__
#include "agl.h"		//	Defined for MacOS applications
#elif defined __LINUX__ || defined __linux__
#include <GL/glx.h>		// Defined for Linux applications
#include <GL/gl.h>
#endif

struct gl_driver_information;

extern void MESG(const char *pszFormat, ...);
extern void DisplaySplashScreen(int sdk);

#ifdef _DEBUG
#define APIDEBUG(str) MESG(__FILE__":"#str"\n")
#define APIBREAK() __asm int 3
#else
#define APIDEBUG(str)
#define APIBREAK()
#endif

#define ASSERT assert

class BIDirect3D8 : public IDirect3D8
{
	public:
		BIDirect3D8();
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(CheckDepthStencilMatch)(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT AdapterFormat, D3DFORMAT RenderTargetFormat, 
		D3DFORMAT DepthStencilFormat);
		STDMETHOD(CheckDeviceFormat)(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT AdapaterFormat, DWORD Usage, D3DRESOURCETYPE RType, D3DFORMAT CheckFormat);
		STDMETHOD(CheckDeviceMultiSampleType)(UINT Adapter, D3DDEVTYPE DeviceType, D3DFORMAT SurfaceFormat, BOOL Windowed, D3DMULTISAMPLE_TYPE MultiSampleType);
		STDMETHOD(CheckDeviceType)(UINT Adapter, D3DDEVTYPE CheckType, D3DFORMAT DisplayFormat, D3DFORMAT BackBufferFormat, BOOL Windowed);
		STDMETHOD(CreateDevice)(UINT Adapter, D3DDEVTYPE DeviceType, HWND hFocusWindow, DWORD BehaviorFlags, D3DPRESENT_PARAMETERS *pPresentationParameters, IDirect3DDevice8 **ppReturnedDeviceInterface);
		STDMETHOD(EnumAdapterModes)(UINT Adapter, UINT Mode, D3DDISPLAYMODE *pMode);
		STDMETHOD_(UINT, GetAdapterCount)();
		STDMETHOD_(UINT, GetAdapterDisplayMode)(UINT Adapter, D3DDISPLAYMODE *pMode);
		STDMETHOD(GetAdapterIdentifier)(UINT Adapter, DWORD Flags, D3DADAPTER_IDENTIFIER8 *pIdentifier);
		STDMETHOD_(HMONITOR, GetAdapterMonitor)(UINT Adapter);
		STDMETHOD_(UINT, GetAdapterModeCount)(UINT Adapter);
		STDMETHOD(GetDeviceCaps)(UINT Adapter, D3DDEVTYPE DeviceType, D3DCAPS8 *pCaps);
		STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
		STDMETHOD(RegisterSoftwareDevice)(void *pInitializeFunction);
		STDMETHOD_(ULONG, Release)();

		gl_driver_information *	m_pGlDevice;
		int						m_refCount;
};


class BIDirect3DDevice8 : public IDirect3DDevice8
{
	public:
		BIDirect3DDevice8(IDirect3D8 *p);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(ApplyStateBlock)(DWORD Token);
		STDMETHOD(BeginScene)();
		STDMETHOD(BeginStateBlock)();
		STDMETHOD(CaptureStateBlock)(DWORD Token);
		STDMETHOD(Clear)(DWORD Count, CONST D3DRECT *pRects, DWORD Flags, D3DCOLOR Color, float Z, DWORD Stencil);
		STDMETHOD(CopyRects)(IDirect3DSurface8 *pSourceSurface, CONST RECT *pSourceRectsArray, UINT cRects, IDirect3DSurface8 *pDestinationSurface, CONST POINT *pDestPointsArray);
		STDMETHOD(CreateAdditionalSwapChain)(D3DPRESENT_PARAMETERS *pPresentationParameters, IDirect3DSwapChain8 **pSwapChain);
		STDMETHOD(CreateCubeTexture)(UINT EdgeLength, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool, IDirect3DCubeTexture8 **ppCubeTexture);
		STDMETHOD(CreateDepthStencilSurface)(UINT Width, UINT Height, D3DFORMAT Format, D3DMULTISAMPLE_TYPE MultiSample, IDirect3DSurface8 **ppSurface);
		STDMETHOD(CreateImageSurface)(UINT Width, UINT Height, D3DFORMAT Format, IDirect3DSurface8 **ppSurface);
		STDMETHOD(CreateIndexBuffer)(UINT Length, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool, IDirect3DIndexBuffer8 **ppIndexBuffer);
		STDMETHOD(CreatePixelShader)(CONST DWORD *pFunction, DWORD *pHandle);
		STDMETHOD(CreateRenderTarget)(UINT Width, UINT Height, D3DFORMAT Format, D3DMULTISAMPLE_TYPE MultiSample, BOOL Lockable, IDirect3DSurface8 **ppSurface);
		STDMETHOD(CreateStateBlock)(D3DSTATEBLOCKTYPE Type, DWORD *pToken);
		STDMETHOD(CreateTexture)(UINT Width, UINT Height, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool, IDirect3DTexture8 **ppTexture);
		STDMETHOD(CreateVertexBuffer)(UINT Length, DWORD Usage, DWORD FVF, D3DPOOL, IDirect3DVertexBuffer8 **ppVertexBuffer);
		STDMETHOD(CreateVertexShader)(CONST DWORD *pDeclaration, CONST DWORD *pFunction, DWORD *pHandle, DWORD Usage);
		STDMETHOD(CreateVolumeTexture)(UINT Width, UINT Height, UINT Depth, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool, IDirect3DVolumeTexture8 **ppVolumeTexture);
		STDMETHOD(DeletePatch)(UINT Handle);
		STDMETHOD(DeletePixelShader)(DWORD Handle);
		STDMETHOD(DrawIndexedPrimitive)(D3DPRIMITIVETYPE type, UINT minIndex, UINT NumVertices, UINT startIndex, UINT primCount);
		STDMETHOD(DrawIndexedPrimitiveUP)(D3DPRIMITIVETYPE type, UINT MinVertexIndex, UINT NumVertexIndices, UINT PrimitiveCount, CONST void *pIndexData, D3DFORMAT IndexDataFormat, CONST void *pVertexStreamZeroData, UINT VertexStreamZeroStride);
		STDMETHOD(DrawPrimitive)(D3DPRIMITIVETYPE PrimitiveTYpe, UINT StartVertex, UINT PrimiveCount);
		STDMETHOD(DrawPrimitiveUP)(D3DPRIMITIVETYPE PrimitiveType, UINT PrimitiveCount, CONST void *pVertexStreamZeroData, UINT VertexStreamZeroStride);
		STDMETHOD(DrawRectPatch)(UINT Handle, CONST float *pNumSegs, CONST D3DRECTPATCH_INFO *pRectPatchInfo);
		STDMETHOD(DrawTriPatch)(UINT Handle, CONST float *pNumSegs, CONST D3DTRIPATCH_INFO *pTriPatchInfo);
		STDMETHOD(EndScene)();
		STDMETHOD(EndStateBlock)(DWORD *pToken);
		STDMETHOD_(UINT, GetAvailableTextureMem)();
		STDMETHOD(GetBackBuffer)(UINT BackBuffer, D3DBACKBUFFER_TYPE Type, IDirect3DSurface8 **ppBackBuffer);
		STDMETHOD(GetClipPlane)(DWORD Index, float *pPlane);
		STDMETHOD(GetClipStatus)(D3DCLIPSTATUS8 *pClipStatus);
		STDMETHOD(GetCreationParameters)(D3DDEVICE_CREATION_PARAMETERS *pParameters);
		STDMETHOD(GetCurrentTexturePalette)(UINT *PaletteNumber);
		STDMETHOD(GetDepthStencilSurface)(IDirect3DSurface8 **ppZStencilSurface);
		STDMETHOD(GetDeviceCaps)(D3DCAPS8 *pCaps);
		STDMETHOD(GetDirect3D)(IDirect3D8 **ppD3D);
		STDMETHOD(GetDisplayMode)(D3DDISPLAYMODE *pMode);
		STDMETHOD(GetFrontBuffer)(IDirect3DSurface8 *pDestSurface);
		STDMETHOD_(void, GetGammaRamp)(D3DGAMMARAMP *pRamp);
		STDMETHOD(GetIndices)(IDirect3DIndexBuffer8 **ppIndexData, UINT *pBaseVertexIndex);
		STDMETHOD(GetInfo)(DWORD DevInfoID, void *pDevInfoStruct, DWORD DevInfoStructSize);
		STDMETHOD(GetLight)(DWORD Index, D3DLIGHT8 *p);
		STDMETHOD(GetLightEnable)(DWORD Index , BOOL *pEnable);
		STDMETHOD(GetMaterial)(D3DMATERIAL8 *pMaterial);
		STDMETHOD(GetPaletteEntries)(UINT PaletteNumber, PALETTEENTRY *pEntries);
		STDMETHOD(GetPixelShader)(DWORD *pHandle);
		STDMETHOD(GetPixelShaderConstant)(DWORD Register, void *pConstantData, DWORD ConstantCount);
		STDMETHOD(GetRasterStatus)(D3DRASTER_STATUS *pRasterStatus);
		STDMETHOD(GetRenderState)(D3DRENDERSTATETYPE State, DWORD *pValue);
		STDMETHOD(GetRenderTarget)(IDirect3DSurface8 **ppRenderTarget);
		STDMETHOD(GetStreamSource)(UINT StreamNumber, IDirect3DVertexBuffer8 **ppStreamData, UINT *pStride);
		STDMETHOD(GetTexture)(DWORD Stage, IDirect3DBaseTexture8 **ppTexture);
		STDMETHOD(GetTextureStageState)(DWORD Stage, D3DTEXTURESTAGESTATETYPE Type, DWORD *pValue);
		STDMETHOD(GetTransform)(D3DTRANSFORMSTATETYPE State, D3DMATRIX *pMatrix);
		STDMETHOD(GetVertexShader)(DWORD *pHandle);
		STDMETHOD(GetVertexShaderConstant)(DWORD Register, void *pConstantData, DWORD ConstantCount);
		STDMETHOD(GetVertexShaderDeclaration)(DWORD Handle, void *pData, DWORD *pSizeOfData);
		STDMETHOD(GetVertexShaderFunction)(DWORD Handle, void *pData, DWORD *pSizeOfData);
		STDMETHOD(GetViewport)(D3DVIEWPORT8 *pViewport);
		STDMETHOD(LightEnable)(DWORD Index, BOOL Enable);
		STDMETHOD(MultiplyTransform)(D3DTRANSFORMSTATETYPE state, CONST D3DMATRIX *pMatrix);
		STDMETHOD(Present)(CONST RECT *pSourceRect, CONST RECT *pDestRect, HWND hDestWindowOverride, CONST RGNDATA *pDirtyRegion);
		STDMETHOD(ProcessVertices)(UINT SrcStartIndex, UINT DestIndex, UINT VertexCount, IDirect3DVertexBuffer8 *pDestBuffer, DWORD Flags);
		STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD(Reset)(D3DPRESENT_PARAMETERS *pPresentationParameters);
		STDMETHOD(ResourceManagerDiscardBytes)(DWORD Bytes);
		STDMETHOD(SetClipPlane)(DWORD Index, CONST float *pPlane);
		STDMETHOD(SetClipStatus)(CONST D3DCLIPSTATUS8 *pClipStatus);
		STDMETHOD(SetCurrentTexturePalette)(UINT PaletteNumber);
		STDMETHOD_(void, SetCursorPosition)(UINT XScreenSpace, UINT YScreenSpace, DWORD Flags);
		STDMETHOD(SetCursorProperties)(UINT XHotSpot, UINT YHotSport, IDirect3DSurface8 *pCursorBitmap);
		STDMETHOD_(void, SetGammaRamp)(DWORD Flags, CONST D3DGAMMARAMP *pRamp);
		STDMETHOD(SetLight)(DWORD Index, CONST D3DLIGHT8 *p);
		STDMETHOD(SetMaterial)(CONST D3DMATERIAL8 *pMaterial);
		STDMETHOD(SetPaletteEntries)(UINT PaletteNumber, CONST PALETTEENTRY *pEntries);
		STDMETHOD(SetPixelShader)(DWORD pHandle);
		STDMETHOD(SetPixelShaderConstant)(DWORD Register, CONST void *pConstantData, DWORD ConstantCount);
		STDMETHOD(SetRenderState)(D3DRENDERSTATETYPE State, DWORD pValue);
		STDMETHOD(SetRenderTarget)(IDirect3DSurface8 *pRenderTarget, IDirect3DSurface8 *pNewZStencil);
		STDMETHOD(SetStreamSource)(UINT StreamNumber, IDirect3DVertexBuffer8 *ppStreamData, UINT pStride);
		STDMETHOD(SetTexture)(DWORD Stage, IDirect3DBaseTexture8 *ppTexture);
		STDMETHOD(SetTextureStageState)(DWORD Stage, D3DTEXTURESTAGESTATETYPE Type, DWORD pValue);
		STDMETHOD(SetTransform)(D3DTRANSFORMSTATETYPE State, CONST D3DMATRIX *pMatrix);
		STDMETHOD(SetVertexShader)(DWORD pHandle);
		STDMETHOD(SetVertexShaderConstant)(DWORD Register, CONST void *pConstantData, DWORD ConstantCount);
		STDMETHOD(SetViewport)(CONST D3DVIEWPORT8 *pViewport);
		STDMETHOD_(BOOL, ShowCursor)(BOOL bShow);
		STDMETHOD(TestCooperativeLevel)();
		STDMETHOD(UpdateTexture)(IDirect3DBaseTexture8 *pSourceTexture, IDirect3DBaseTexture8 *pDestinationTexture);
		STDMETHOD(ValidateDevice)(DWORD *pNumPasses);
		STDMETHOD(DeleteStateBlock)(DWORD Token);
		STDMETHOD(DeleteVertexShader)(DWORD Handle);
		STDMETHOD(SetIndices)(IDirect3DIndexBuffer8* pIndexData, UINT BaseVertexIndex);
		STDMETHOD(GetPixelShaderFunction)(DWORD Handle, void *pData, DWORD *pSizeOfData);
	private:

		DWORD GetCurrentFVF() const;
		DWORD GetCurrentFVFSize() const;
		CONST VOID *GetCurrentVertexData() const;
		CONST VOID *GetCurrentIndexData() const;

		DWORD						m_RenderState[256];		
		IDirect3D8				*	m_pIDirect3D8;
		D3DMATERIAL8				m_d3dmaterial;
		D3DVIEWPORT8				m_viewport;
		D3DGAMMARAMP				m_pRamp;
		D3DDISPLAYMODE				m_pDisplayMode;
		D3DDEVICE_CREATION_PARAMETERS m_pCreationParameters;
		D3DLIGHT8					m_pLight[16];
		BOOL						m_bLight[16];
		UINT						m_PaletteNumber;
		float						m_pPlane[32];
		D3DCLIPSTATUS8				m_pClipStatus;
		DWORD						m_pVertexShaderHandle;
		D3DMATRIX					m_MatView, m_PrjView, m_WldView;
		IDirect3DBaseTexture8 *		m_pTexture[8];
		IDirect3DSurface8 *			m_pZStencilSurface;
		IDirect3DSurface8 *			m_pBackBuffer;		
		UINT						m_nBaseVertexIndex;

		IDirect3DVertexBuffer8	*	m_pCurrentVertexBuffer;
		UINT						m_nCurrentVertexStride;
		IDirect3DIndexBuffer8	*	m_pCurrentIndexBuffer;

		DWORD						m_nCurrentStreamFVF;
		DWORD						m_nCurrentStreamFVFSize;
		int							m_refCount;


};


class BIDirect3DBaseTexture8 : public IDirect3DBaseTexture8
{
	public:
		BIDirect3DBaseTexture8(IDirect3DDevice8 *pDevice);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(FreePrivateData)(REFGUID refguid);
		STDMETHOD(GetDevice)(IDirect3DDevice8 **ppDevice);
		STDMETHOD_(DWORD, GetLevelCount)();
		STDMETHOD_(DWORD, GetLOD)();
		STDMETHOD_(DWORD, GetPriority)();
		STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
		STDMETHOD_(D3DRESOURCETYPE, GetType)();
		STDMETHOD_(void, PreLoad)();
		STDMETHOD(QueryInterface)(REFIID, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD_(DWORD, SetLOD)(DWORD Lodnew);
		STDMETHOD_(DWORD, SetPriority)(DWORD PriorityNew);
		STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
	protected:
		IDirect3DDevice8	*	m_pDevice;
		int						m_refCount;
};


class BIDirect3DTexture8 : public IDirect3DTexture8
{
	public:	
		BIDirect3DTexture8(IDirect3DDevice8 *m_pDevice, UINT Width, UINT Height, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(FreePrivateData)(REFGUID refguid);
		STDMETHOD(GetDevice)(IDirect3DDevice8 **ppDevice);
		STDMETHOD_(DWORD, GetLevelCount)();
		STDMETHOD_(DWORD, GetLOD)();
		STDMETHOD_(DWORD, SetLOD)(DWORD Lodnew);
		STDMETHOD_(DWORD, GetPriority)();
		STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
		STDMETHOD_(D3DRESOURCETYPE, GetType)();
		STDMETHOD_(void, PreLoad)();
		STDMETHOD(QueryInterface)(REFIID, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD_(DWORD, SetPriority)(DWORD PriorityNew);
		STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
		
		STDMETHOD(GetLevelDesc)(UINT Level, D3DSURFACE_DESC *pDesc);
		STDMETHOD(LockRect)(UINT Level, D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags);
		STDMETHOD(GetSurfaceLevel)(UINT Level, IDirect3DSurface8 **ppSurfaceLevel);
		STDMETHOD(UnlockRect)(UINT Level);
	
		STDMETHOD(AddDirtyRect)(CONST RECT *pDirtyRect);


		void					BindTexture();

		IDirect3DDevice8 *		m_pDevice;	
		
		UINT					m_Width;
		UINT					m_Height;
		UINT					m_Levels;
		DWORD					m_Usage;
		D3DFORMAT				m_Format;
		D3DPOOL					m_Pool;
		DWORD					m_Lod;
		DWORD					m_Priority;
		GLuint					m_GlHandle;
		int						m_refCount;
		GLenum					m_InternalFmt;
		GLenum					m_TextureFmt;
		void			*		m_pLocalImage;
		int						m_Pitch;
};



class BIDirect3DIndexBuffer8 : public IDirect3DIndexBuffer8
{
	public:		
		BIDirect3DIndexBuffer8(IDirect3DDevice8 *pDevice, UINT Length, DWORD Usage, DWORD FVF, D3DPOOL pool);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(FreePrivateData)(REFGUID refguid);
		STDMETHOD(GetDesc)(D3DINDEXBUFFER_DESC *pDesc);
		STDMETHOD(GetDevice)(IDirect3DDevice8 **pDevice);
		STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
		STDMETHOD_(D3DRESOURCETYPE, GetType)();
		STDMETHOD(Lock)(UINT OffsetToLock, UINT SizeToLock, BYTE **ppbData, DWORD Flags);
		STDMETHOD_(void, PreLoad)();
		STDMETHOD_(DWORD, GetPriority)();
		STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD_(DWORD, SetPriority)(DWORD PriorityNew);
		STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
		STDMETHOD(Unlock)();

		
		BYTE	*m_pIndex;
	private:
		
		IDirect3DDevice8	*	m_pDevice;
		UINT					m_Length;
		DWORD					m_Usage;
		DWORD					m_FVF;
		DWORD					m_Priority;
		D3DPOOL					m_pool;
		int						m_refCount;
};


class BIDirect3DSurface8 : public IDirect3DSurface8
{
	public:
		BIDirect3DSurface8(IDirect3DDevice8 *pDevice, int Width, int Height, D3DFORMAT fmt, D3DMULTISAMPLE_TYPE mstype = D3DMULTISAMPLE_NONE, BOOL Lockable = FALSE);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(FreePrivateData)(REFGUID refguid);
		STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
		STDMETHOD(LockRect)(D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags);
		STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
		STDMETHOD(UnlockRect)();
		STDMETHOD(GetDevice)(IDirect3DDevice8 **ppDevice);
		STDMETHOD(GetContainer)(REFIID iid, void **ppContainer);
		STDMETHOD(GetDesc)(D3DSURFACE_DESC *pDesc);
	private:
		IDirect3DDevice8 *		m_pDevice;
		int						m_Width;
		int						m_Height;
		D3DFORMAT				m_Format;
		D3DMULTISAMPLE_TYPE		m_mstype;
		int						m_refCount;
		

};

class BIDirect3DSwapChain8 : public IDirect3DSwapChain8
{
	public:
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(GetBackBuffer)(UINT BackBuffer, D3DBACKBUFFER_TYPE Type, IDirect3DSurface8 **ppBackBuffer);
		STDMETHOD(Present)(CONST RECT *pSourceRect, CONST RECT *pDestRect, HWND hDestWindowOverride, CONST RGNDATA *pDirtyRegion);
		STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
};

class BIDirect3DVertexBuffer8 : public IDirect3DVertexBuffer8
{
	public:
		BIDirect3DVertexBuffer8(IDirect3DDevice8 *pDevice, UINT Length, DWORD Usage, DWORD FVF, D3DPOOL pool);
		STDMETHOD(FreePrivateData)(REFGUID refguid);
		STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
		STDMETHOD_(void, PreLoad)();
		STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(GetDevice)(IDirect3DDevice8 **ppDevice);
		STDMETHOD_(DWORD, SetPriority)(DWORD PriorityNew);
		STDMETHOD_(DWORD, GetPriority)();
		STDMETHOD_(D3DRESOURCETYPE, GetType)();
		STDMETHOD(Lock)(UINT OffsetToLock, UINT SizeToLock, BYTE **ppbData, DWORD Flags);
		STDMETHOD(Unlock)();
		STDMETHOD(GetDesc)(D3DVERTEXBUFFER_DESC *pDesc);

		BYTE		*			m_pVtxBuffer;
	private:
		IDirect3DDevice8 *		m_pDevice;
		UINT					m_Length;
		DWORD					m_Usage;
		DWORD					m_FVF;
		D3DPOOL					m_pool;
		int						m_refCount;

		
};


class BIDirect3DVolumeTexture8 : public IDirect3DVolumeTexture8
{
	public:
		BIDirect3DVolumeTexture8(IDirect3DDevice8 *pDevice, INT Width, UINT Height, UINT Depth, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool);
		STDMETHOD(FreePrivateData)(REFGUID refguid);
		STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
		STDMETHOD_(void, PreLoad)();
		STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(GetDevice)(IDirect3DDevice8 **ppDevice);
		STDMETHOD_(DWORD, SetPriority)(DWORD PriorityNew);
		STDMETHOD_(DWORD, GetPriority)();
		STDMETHOD_(DWORD, SetLOD)(DWORD Lodnew);
		STDMETHOD_(DWORD, GetLevelCount)();
		STDMETHOD_(DWORD, GetLOD)();
		STDMETHOD_(D3DRESOURCETYPE, GetType)();
		STDMETHOD(GetLevelDesc)(UINT Level, D3DVOLUME_DESC *pDesc);
		STDMETHOD(GetVolumeLevel)(UINT Level, IDirect3DVolume8**ppVolume);
		STDMETHOD(LockBox)(UINT Level, D3DLOCKED_BOX *pLockedVolume, CONST D3DBOX *pBox, DWORD Flags);
		STDMETHOD(UnlockBox)(UINT Level);
		STDMETHOD(AddDirtyBox)(CONST D3DBOX *pDirtyBox);


	private:
		IDirect3DDevice8	*	m_pDevice;
		UINT					m_Width;
		UINT					m_Height;
		UINT					m_Depth;
		UINT					m_Levels;
		DWORD					m_Usage;
		D3DFORMAT				m_Format;
		D3DPOOL					m_Pool;
		DWORD					m_Lod;
		DWORD					m_Priority;
		int						m_refCount;
};

class BIDirect3DVolume8 : public IDirect3DVolume8
{
	public:
		//
	STDMETHOD(FreePrivateData)(REFGUID refguid);
	STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
	STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
	STDMETHOD_(ULONG, Release)();
	STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
};


class BIDirect3DCubeTexture8 : public IDirect3DCubeTexture8
{
	public:
		BIDirect3DCubeTexture8(IDirect3DDevice8 *pDevice, UINT EdgeLength, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool);
		STDMETHOD(AddDirtyRect)(D3DCUBEMAP_FACES FaceType, CONST RECT *pDirtyRect);
		STDMETHOD_(ULONG, AddRef)();
		STDMETHOD(FreePrivateData)(REFGUID refguid);
		STDMETHOD(GetDevice)(IDirect3DDevice8 **ppDevice);
		STDMETHOD(GetCubeMapSurface)(D3DCUBEMAP_FACES FaceType, UINT Level , IDirect3DSurface8 **ppCubeMapSurface);
		STDMETHOD_(DWORD, GetLOD)();
		STDMETHOD_(DWORD, GetLevelCount)();
		STDMETHOD_(DWORD, GetPriority)();
		STDMETHOD(GetLevelDesc)(UINT Level, D3DSURFACE_DESC *pDesc);
		STDMETHOD(GetPrivateData)(REFGUID refguid, void *pData, DWORD *pSizeOfData);
		STDMETHOD(LockRect)(D3DCUBEMAP_FACES FaceType, UINT Level, D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags);
		STDMETHOD_(D3DRESOURCETYPE, GetType)();
		STDMETHOD_(DWORD, SetLOD)(DWORD LODNew);
		STDMETHOD_(void, PreLoad)();
		STDMETHOD(QueryInterface)(REFIID, void **ppvObj);
		STDMETHOD_(ULONG, Release)();
		STDMETHOD_(DWORD, SetPriority)(DWORD PriorityNew);
		STDMETHOD(SetPrivateData)(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags);
		STDMETHOD(UnlockRect)(D3DCUBEMAP_FACES FaceType, UINT Level);
	private:
		IDirect3DDevice8	*	m_pDevice;
		UINT					m_EdgeLength;
		UINT					m_Levels;
		DWORD					m_Usage;
		D3DFORMAT				m_Format;
		D3DPOOL					m_Pool;
		DWORD					m_Lod;
		DWORD					m_Priority;
		int						m_refCount;
};

extern const char *GL_GetErrorString(GLenum err);
#ifdef _DEBUG
#define GL_DEBUG_TRY() glGetError()
#define GL_DEBUG_CATCH() { GLenum __err = glGetError(); if (__err) MESG("%s: %s line %d ", __FILE__, GL_GetErrorString(__err), __LINE__); }
//#define GL_DEBUG_CATCH() SYS_ASSERT(GL_NO_ERROR == glGetError())
#else
#define GL_DEBUG_TRY()
#define GL_DEBUG_CATCH()
#endif



#endif // _D3DPRJ_HH