/*==========================================================================;
 *
 *  Copyright (C) 1995-2001 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       idirect3dtexture8.cpp
 *  Content:    Direct3D capabilities include file
 *
 ***************************************************************************/
#include "d3dprj.h"

GLenum D3D8ToGLEnum(D3DFORMAT fmt)
{
	return GL_RGBA;
}

class BIDirect3DSurface8_itex : public BIDirect3DSurface8
{
public:
	BIDirect3DSurface8_itex(BIDirect3DTexture8 *pTex) 
		: BIDirect3DSurface8(pTex->m_pDevice, pTex->m_Width, pTex->m_Height, pTex->m_Format, D3DMULTISAMPLE_NONE, TRUE), m_pTex(pTex)
	{
		pTex->AddRef();
	}

	STDMETHOD(LockRect)(D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags)
	{
		return m_pTex->LockRect(0, pLockedRect, pRect, Flags);
	}

	STDMETHOD_(ULONG, Release)()
	{
		return m_pTex->Release();
	}

	STDMETHOD(UnlockRect)()
	{
		return m_pTex->UnlockRect(0);
	}

	BIDirect3DTexture8	*	m_pTex;
};


BIDirect3DTexture8::BIDirect3DTexture8(IDirect3DDevice8 *pDevice, UINT Width, UINT Height, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool) :
m_pDevice(pDevice), m_Width(Width), m_Height(Height), m_Levels(Levels), m_Usage(Usage), m_Format(Format), m_Pool(Pool), m_refCount(0),
m_InternalFmt(0), m_TextureFmt(0), m_pLocalImage(0), m_Pitch(0)
{	
	glGenTextures(1, &m_GlHandle);	
	glBindTexture(GL_TEXTURE_2D, m_GlHandle);

	if (Format == D3DFMT_P8)
	{
		m_InternalFmt = GL_COLOR_INDEX8_EXT;
		m_TextureFmt = GL_COLOR_INDEX;	
		m_Pitch = m_Width;
	}
	else
	if (Format == D3DFMT_R5G6B5)
	{
		m_InternalFmt = GL_RGB5;
		m_TextureFmt = GL_BGR_EXT;	
		m_Pitch = m_Width * 2; 
	}
	else
	if (Format == D3DFMT_A8R8G8B8)
	{
		m_InternalFmt = GL_RGBA8;
		m_TextureFmt = GL_BGRA_EXT;	
		m_Pitch = m_Width * 4;
	}
	else
	if (Format == D3DFMT_X8R8G8B8)
	{
		m_InternalFmt = GL_RGBA8;
		m_TextureFmt = GL_BGRA_EXT;	
		m_Pitch = m_Width * 4;
	}
	else
	{
		// ?
	}
	m_pLocalImage = new char[m_Width * m_Pitch];
	ASSERT(m_pLocalImage);
	memset(m_pLocalImage, 0xff, m_Width * m_Pitch);
}

void
BIDirect3DTexture8::BindTexture()
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_GlHandle);
}

ULONG
BIDirect3DTexture8::AddRef()
{
	APIDEBUG(AddRef);
	m_refCount++;
	return 0;
}

HRESULT 
BIDirect3DTexture8::FreePrivateData(REFGUID refguid)
{
	APIDEBUG(FreePrivateData);
	return D3D_OK;
}

HRESULT
BIDirect3DTexture8::GetDevice(IDirect3DDevice8 **ppDevice)
{
	APIDEBUG(GetDevice);
	*ppDevice = m_pDevice;
	m_pDevice->AddRef();
	return D3D_OK;
}

DWORD
BIDirect3DTexture8::GetLevelCount()
{
	APIDEBUG(GetLevelCount);
	return m_Levels;
}

DWORD
BIDirect3DTexture8::GetLOD()
{
	APIDEBUG(GetLOD);
	return m_Levels;
}

DWORD
BIDirect3DTexture8::GetPriority()
{
	APIDEBUG(GetPriority);
	return m_Priority;
}

HRESULT
BIDirect3DTexture8::GetPrivateData(REFGUID refguid, void *pData, DWORD *pSizeOfData)
{
	// TODO: Handle this com interface
	APIDEBUG(GetPrivateData);
	*pSizeOfData = 0;
	return D3D_OK;
}

D3DRESOURCETYPE
BIDirect3DTexture8::GetType()
{
	APIDEBUG(GetType);
	return D3DRTYPE_TEXTURE;
}

void
BIDirect3DTexture8::PreLoad()
{
	APIDEBUG(PreLoad);
	return ;
}

HRESULT
BIDirect3DTexture8::QueryInterface(REFIID, void **ppvObj)
{
	// TODO: Handle this com interface - should always return an error
	APIDEBUG(QueryInterface);
	*ppvObj = 0;
	return D3D_OK;
}

ULONG
BIDirect3DTexture8::Release()
{
	APIDEBUG(Release);
	m_refCount--;
	if (m_refCount<0)
	{
		if (m_GlHandle)
		{
			glDeleteTextures(1, &m_GlHandle);
			m_GlHandle = 0;
			delete[] m_pLocalImage;
		}
	}
	return D3D_OK;
}

DWORD
BIDirect3DTexture8::SetPriority(DWORD PriorityNew)
{
	APIDEBUG(SetPriority);
	m_Priority = PriorityNew;	
	float v = (float)PriorityNew / 255.f;
	glPrioritizeTextures(1, &m_GlHandle, &v);
	return D3D_OK;
}

HRESULT
BIDirect3DTexture8::SetPrivateData(REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags)
{
	return D3D_OK;
}

HRESULT BIDirect3DTexture8::GetLevelDesc(UINT Level, D3DSURFACE_DESC *pDesc)
{	
	APIDEBUG(GetLevelDesc);
	if (Level>m_Levels)
		return D3DERR_INVALIDCALL;

	memset(pDesc, 0, sizeof(pDesc));
	pDesc->Format = m_Format;
	pDesc->Height = m_Height>>Level;
	pDesc->Width = m_Width>>Level;

	return D3D_OK;
}

HRESULT BIDirect3DTexture8::LockRect(UINT Level, D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags)
{
	APIDEBUG(LockRect);
	pLockedRect->pBits = m_pLocalImage;
	pLockedRect->Pitch = m_Pitch;
	
	return D3D_OK;
}

HRESULT BIDirect3DTexture8::GetSurfaceLevel(UINT Level, IDirect3DSurface8 **ppSurfaceLevel)
{
	APIDEBUG(GetSurfaceLevel);
	if ((Level>m_Levels)||(!ppSurfaceLevel))
		return D3DERR_INVALIDCALL;

	*ppSurfaceLevel = new BIDirect3DSurface8_itex(this);
	return D3D_OK; // not supported
}

HRESULT BIDirect3DTexture8::UnlockRect(UINT Level)
{
	APIDEBUG(UnlockRect);
	GL_DEBUG_TRY();

	glEnable( GL_TEXTURE_2D);
	glBindTexture( GL_TEXTURE_2D, m_GlHandle );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	glPixelStorei( GL_PACK_ROW_LENGTH, 0);
	glPixelStorei( GL_PACK_SKIP_PIXELS, 0);		
	
	//memset(m_pLocalImage, rand()&0xff, m_Width * m_Pitch);


	glTexImage2D( GL_TEXTURE_2D,
		0, 						    // base level
		m_InternalFmt,		 		// number of color components in the texture
		m_Width, 
		m_Height,
		0, 							// border
		m_TextureFmt, 				// texture format
		GL_UNSIGNED_BYTE, 			// texture type
		m_pLocalImage
	);


	GL_DEBUG_CATCH();

	
	return D3D_OK;
}

HRESULT BIDirect3DTexture8::AddDirtyRect(CONST RECT *pDirtyRect)
{
	APIDEBUG(AddDirtyRect);
	return D3D_OK;
}

DWORD BIDirect3DTexture8::SetLOD(DWORD lod)
{	
	APIDEBUG(SetLOD);
	m_Lod = lod;
	return m_Lod;
}
